# This Praat script is provided to illustrate the semi-automatic segmentation of a 
# Long Sound File into its components.
#
# There are two parts.
# 
# PART 1: generate a TextGrid, which is presented to the user for consideration.
#  The user should enter an 'x' in any interval that is not to be considered
#  All remaining intervals will be interpreted as a sequence of pairs, consisting
#  of sound + pause
#
# PART 2: Pairs of intervals, interpreted as sound+pause, will be written out as single sound
# files in ../recordings.
# See the documentation in this folder for further information
#
# Both parts are derived from functions authored by Mietta Lennes, and made available  
# within the  Speech Corpus Toolkit for Praat available here: 
#                           http://www.helsinki.fi/~lennes/praat-scripts/
# Please acknowledge this source in any further derivative work.
# This derivative is authored by Fred Cummins (2013)

# USE:
#  I assume that you have a source recording consisting of multiple repetitions of the same
#  utterance.  In the distribution provided, this is 'sourceRecording.wav'.
#  Open this script in Praat and run it.  Provide the path to your source recording.
#  You will then be presented with a waveform editor and a text grid.  
#  Enter an 'x' in each interval you wish the programme to ignore.  All remaining 
#  intervals will be interpreted as Utterance+Pause pairs.  When you click 'continue',
#  These will appear as single utterance sound files in the 'recordings' folder.

form Provide the path to your folder
   comment  Please provide the path to the folder you are using
   sentence Path /Users/fred/Codedev
   comment  Please provide the name of the folder
   sentence Folder UtteranceToUtteranceVariability
   comment Please provide the name of the source sound file
   sentence Source sourceRecording.wav
endform

### PART 1 ####
### This part of the script is taken from Mietta Lennes.  The original script is part of 
### the Speech Corpus Toolkit for Praat
### available here: http://www.helsinki.fi/~lennes/praat-scripts/
#############

Open long sound file... 'path$'/'folder$'/'source$'
soundname$ = selected$ ("LongSound")

starting_time = 0
finishing_time = 0
time_step = 0
boundary_placement = 2
margin = 0.05
window_size = 20
minimum_pitch = 80
maximum_intensity = 59
minimum_duration = 0.2

soundname$ = selected$ ("LongSound")
To TextGrid... sentence 

select TextGrid 'soundname$'
	endofsound = Get finishing time
select LongSound 'soundname$'
	pausenumber = 0
	duration = 0
	count = 0
	loops = 0
	pauses_found = 0
	windowstart = 0
	windowend = 0
	frame = 0
	frames = 0
	time = 0
	intensity = 0
	pausedetected = 0
	pausestart = 0
	pauseend = 0
	pausenumber = 0
	halfpause = 0

if 'finishing_time' < 0
exit Finishing time must be greater than or equal to zero! (If you give a zero, the whole LongSound will be analysed.)
endif
if finishing_time = 0
finishing_time = endofsound
endif
#******************************************************************************************************
# BEGIN
#******************************************************************************************************
# DIVIDE LONGSOUND INTO SHORTER PERIODS AND LOOP THROUGH EACH
duration = finishing_time - starting_time
#--------------------------------------------------------------------------------------------------
# Default number of loops is 1
loops = 1
# but if the period to be analysed is longer than 60 seconds, it will be divided into 60-second
# periods for which the analysis is made:
if duration > window_size
loops = ceiling ((duration/window_size))
endif
#--------------------------------------------------------------------------------------------------
# START LOOPING THROUGH SHORT WINDOWS HERE
count = 1
latest_endboundary = 0
while count <= loops

	# Create a window of the LongSound and extract it for analysis
	windowstart = starting_time + ((count - 1) * window_size)
	windowend = starting_time + (count * window_size)
	if windowend > endofsound
	windowend = endofsound
	endif
	if windowend > finishing_time
	windowend = finishing_time
	endif
	select LongSound 'soundname$'
	Extract part... windowstart windowend yes
	windowname$ = "Window_" + "'count'" + "_of_" + "'loops'"
	echo Analysing Intensity window 'count' of 'loops'

	Rename... 'windowname$'
	#--------------------------------------------------------------------------------------------------
	# CALCULATE INTENSITY
	To Intensity... minimum_pitch time_step
	frames = Get number of frames
	#--------------------------------------------------------------------------------------------------
	# Check the pause criteria
	pauseend = 0
	frame = 1
		#--------------------------------------------------------------------------------------------------
		# Loop through all frames in the Intensity object:
		while frame <= frames
			select Intensity 'windowname$'
			intensity = Get value in frame... frame
			time = Get time from frame... frame
				if intensity > maximum_intensity
					# If the end of an earlier detected possible pause has been reached:
					if pausedetected = 1
						if frame - 1 < 1
						pauseend = windowstart
						else
						pauseend = Get time from frame... (frame - 1)
						endif
						pausedetected = 0
					endif
				    # If below intensity limit, a possible new pause is started if one hasn't been detected yet:
				    elsif pausedetected = 0
						pausestart = Get time from frame... frame
						pauseend = 0
						pausedetected = 1
						pausenumber = pausenumber + 1
				# If a detected pause just continues, do nothing special.
				endif
			#--------------------------------------------------------------------------------------------------
			# IF PAUSE CRITERIA ARE FULFILLED, ADD A BOUNDARY OR TWO TO TEXTGRID
			if pauseend > 0
				pauseduration = pauseend - pausestart
				if pauseduration >= minimum_duration
					select TextGrid 'soundname$'
					halfpause = pauseduration / 2
						if boundary_placement = 1
							boundary = pausestart + halfpause
							call BoundaryCheck
							if boundaryexists = 0
								Insert boundary... 1 boundary
								latest_endboundary = boundary
							endif
						else
							boundary = 0
							if pauseduration >= (2 * margin)
								if pausestart > margin
									boundary = pausestart + margin
									call BoundaryCheck
									if boundaryexists = 0 and boundary > latest_endboundary
										Insert boundary... 1 boundary
									endif
									#If the pause overlaps with the preceding pause, do a merge:
									if boundary = latest_endboundary
										Remove boundary at time... 1 boundary
									endif
								endif
								
								boundary = pauseend - margin
								call BoundaryCheck
								if boundaryexists = 0 and boundary > latest_endboundary
									Insert boundary... 1 boundary
									latest_endboundary = boundary
								endif
							else
								if pauseend < (endofsound - margin)
									boundary = pausestart + halfpause
									call BoundaryCheck
									if boundaryexists = 0 and boundary > latest_endboundary
										Insert boundary... 1 boundary
										latest_endboundary = boundary
									endif
								endif
							endif
						endif
					pauseend = 0
					pauses_found = pauses_found + 1
				endif
			endif
			frame = frame + 1
			# When all frames in the intensity analysis have been looked at, end the frame loop.
		endwhile
		#--------------------------------------------------------------------------------------------------
	select Sound 'windowname$'
	Remove
	select Intensity 'windowname$'
	Remove
	# END LOOPING THROUGH WINDOWS HERE
	count = count + 1
endwhile


echo Ready! 

#******************************************************************************************************

procedure BoundaryCheck
# This procedure checks whether a boundary already exists at a given time (in tier 1).
# Added 23.1.2006
	tmpint = Get interval at time... 1 boundary
	tmpstart = Get starting point... 1 tmpint
	if tmpstart <> boundary
		boundaryexists = 0
	else
		boundaryexists = 1
	endif
endproc

select LongSound 'soundname$'
plus TextGrid 'soundname$'
View & Edit

beginPause("Check pauses")
comment("Enter x where you want an interval to be skipped")
clicked = endPause("Continue", "Abort", 1)

### PART 2
###

tier = 1
start_from = 1
end_at = 0
start_numbering_at = 1

select TextGrid 'soundname$'
numberOfIntervals = Get number of intervals... tier

if start_from > numberOfIntervals
	exit There are not that many intervals in the IntervalTier!
endif
if end_at > numberOfIntervals
	end_at = numberOfIntervals 
endif
if end_at = 0
	end_at = numberOfIntervals 
endif

# Default values for variables
files = 0
intervalstart = 0
intervalend = 0
interval = 1
repetition = 0
intname$ = ""
intervalfile$ = ""
endoffile = Get finishing time

# Loop through all intervals in the selected tier of the TextGrid
interval = start_from
while interval < end_at
	select TextGrid 'soundname$'
	intname$ = ""
	intname$ = Get label of interval... tier interval

	check = 0

	if intname$ = "x" 
		check = 1
                 interval = interval + 1
	else 
		repetition = repetition + 1
		intervalstart = Get starting point... tier interval 
                nextinterval = interval + 1
		intervalend = Get end point... tier nextinterval
	
		select LongSound 'soundname$'
		Extract part... intervalstart intervalend no
		# The name of the sound file then consists of these elements:
		intervalfile$ = "'path$'" + "/" + "'folder$'" + "/recordings/sample" + "_'repetition'" + ".wav"
		Write to WAV file... 'intervalfile$'
		Remove
                interval = interval + 2
	endif
endwhile

select all
Remove
